// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI12WRCIEPE/24DSI12WRCIEPE_linux_1.x.x.x_gsc_dn/utils/util_ch_grp_1_src.c $
// $Rev: 52535 $
// $Date: 2023-03-02 14:55:56 -0600 (Thu, 02 Mar 2023) $

// 24DSI12WRCIEPE: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi12wi_ch_grp_1_src
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI12WI_IOCTL_CH_GRP_1_SRC service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi12wi_ch_grp_1_src(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Group 1 Clock Source", index);

	ret		= dsi12wi_ioctl(fd, DSI12WI_IOCTL_CH_GRP_1_SRC, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case DSI12WI_CH_GRP_SRC_RATE_GEN:

			ptr	= "Rate Generator";
			break;

		case DSI12WI_CH_GRP_SRC_EXTERN:

			ptr	= "External";
			break;

		case DSI12WI_CH_GRP_SRC_DIR_EXTERN:

			ptr	= "Direct External";
			break;

		case DSI12WI_CH_GRP_SRC_DISABLE:

			ptr	= "Disable";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


